# -*- coding: utf-8 -*-
# !/usr/bin/python
# Create Date 2017/9/29 0029
import os
import json
import xlrd
import chardet
from demo_xml import demo_xml
try:
    import csv
except:
    pass
__author__ = 'huohuo'
if __name__ == "__main__":
    pass


class File:

    def __init__(self, base_dir=''):
        self.__description__ = '文件相关， 包括读文件，写文件， 下载文件'
        self.base_dir = base_dir

    def read(self, file_name, sheet_name='', read_type='r', dict_name='', to_json=True, **kwargs):
        file_type = file_name.split('.')[-1]
        url = os.path.join(self.base_dir, dict_name, file_name)
        if os.path.exists(url) is False:
            print 'File not exists, %s. Please check.' % url
            return None
        if file_type in ['xlsx', 'xls']:
            data = xlrd.open_workbook(url)
            return data.sheet_by_name(sheet_name)

        f = open(url, read_type)
        text = f.read()
        f.close()
        if file_type in ['csv', 'tsv'] or 'sep' in kwargs:
            encoding = chardet.detect(text)['encoding']
            sep = ',' if file_type == 'csv' else '\t'
            if 'sep' in kwargs:
                sep = kwargs['sep']
            delimiter = {'delimiter': sep}
            f = open(url, read_type)
            csv.register_dialect('my_dialect', **delimiter)
            cons = csv.reader(f, 'my_dialect')
            items = []
            keys = next(cons)
            for c in cons:
                if to_json:
                    item = {}
                    for i in range(len(keys)):
                        item[keys[i]] = c[i].decode(encoding).encode('utf-8')
                    items.append(item)
                else:
                    items.append([x.decode(encoding).encode('utf-8') for x in c])
            csv.unregister_dialect('my_dialect')
            return items
        if file_type == 'json':
            return json.loads(text)
        return text

    def read_message(self, file_name, file_type='json', sheet_name='', read_type='r', developer='huohuo', message='success'):
        error_message = u'【开发者】：%s\n' % developer
        error_message += u'【文件类型】：%s\n' % file_type
        if file_type in ['xlsx', 'xls']:
            error_message += u'【sheet_name】：%s\n' % sheet_name
        error_message += u'【消息】：%s\n' % message
        error_message += u'【文件名】：\n%s\n' % file_name

    def write(self, file_name, data):
        file_type = file_name.split('.')[-1]
        url = os.path.join(self.base_dir, file_name)
        f = open(url, "w")
        if file_type == 'json':
            try:
                f.write(json.dumps(data, sort_keys=True, indent=4, ensure_ascii=False))
            except:
                f.write(json.dumps(data, sort_keys=True, indent=4, ensure_ascii=True))
        else:
            f.write(data)
        f.close()
        return 5

    def download(self, pkg_parts, file_name):
        temp_data = demo_xml.replace('<pkg:part id="pkg_parts"></pkg:part>', pkg_parts)
        temp_data = temp_data.replace('\n', '')
        return self.write(file_name, temp_data)

    def get_filt_list(self, sample_type, basi_path, f, pre, basi_dir=""):
        file_list = []
        if pre is not None:
            for dirpath, dirnames, filenames in os.walk(os.path.join(basi_dir, pre)):
                if dirpath != os.path.join(basi_path, pre):
                    break
                for dir_name in dirnames:
                    file_dict = self.get_file_item(basi_dir, dirpath, dir_name, f=='/')
                    file_dict["type"] = "文件夹"
                    file_dict["size"] = None
                    file_list.append(file_dict)
                for file_name in filenames:
                    file_type = self.get_file_type(sample_type, file_name)
                    file_dict = self.get_file_item(basi_dir, dirpath, file_name, f=='/')
                    file_dict["type"] = file_type
                    try:
                        file_size = os.path.getsize(file_dict['full_path'])
                        file_dict["size"] = get_file_size(file_size)
                    except Exception, e:
                        file_dict["size"] = ''
                        pass
                        # print file_path
                        # print e.args
                    if file_type != "":
                        file_list.append(file_dict)
        return file_list

    def get_file_size(self, size):
        if size > 1024 ** 3:
            return '%.02fGB' % (float(size) / 1024 ** 3)
        elif size > 1024 ** 2:
            return '%.02fMB' % (float(size) / 1024 ** 2)
        elif size > 1024:
            return '%0.2fKB' % (float(size) / 1024)
        else:
            return '%0.2fB' % (float(size))

    def get_file_item(self, basi_dir, dirpath, relative_path, is_relative):
        item = {}
        full_path = os.path.join(dirpath, relative_path)
        file_url = full_path
        file_text = relative_path
        if is_relative:
            file_url = full_path.replace(basi_dir, "")
            file_text = relative_path.replace(full_path, '')
        file_array = dirpath.replace(basi_dir, "").split(os.sep)
        item['full_path'] = full_path
        item["url"] = file_url.strip()
        item["text"] = file_text.strip()
        item["len"] = len(file_array)
        item["array"] = file_array
        return item

    def get_file_type(self, sample_type, file_name):
        postfix = ['.%s' % sample_type]
        file_type = sample_type
        if sample_type == 'vcf':
            postfix.append('.%s.gz' % sample_type)
        elif sample_type == "fastq":
            if file_name.endswith('.bed'):
                file_type = 'bed'
                postfix = ['.bed']
            else:
                postfix.extend(['.fq', '.fq.gz', '.fastq.gz'])
        for p in postfix:
            if file_name.endswith(p):
                return file_type
        return ''

    def is_fastq(self, file_name):
        postfix = ['.fastq', '.fq', '.fastq.gz', '.fq.gz']
        for p in postfix:
            if file_name.endswith(p):
                return True
        return False

    # 对文件进行排序 返回值大于零，则交换x、y,否则不交换
    def compare_file(self, x, y):
        '''
        原则：
        1. 文件夹在前
        2. fastq 在前 bed 在后
        3. 子文件夹嵌套层数越多越靠前
        '''
        if x['type'] == '文件夹' and y['type'] != '文件夹':
            return -1
        if y['type'] == '文件夹' and x['type'] != '文件夹':
            return 1
        if x['type'] == 'bed' and y['type'] == 'fastq':
            if x['text'][:3] == y['text'][:3]:
                return 1
        if x['len'] != y['len']:
            return y['len'] - x['len']
        return cmp(x['url'], y['url'])



# -*- coding: utf-8 -*-
# !/usr/bin/python
# Create Date 2018/7/10 0010
__author__ = 'huohuo'
import argparse, os
parser = argparse.ArgumentParser(
    description='Converts a CSV file to JSON',
    epilog='To avoid ambiguity put the list following flags -u, -n, -r and -a into quotes. ' +
           'Alternatively, supply infile (and optional outfile) before listing the flags.'
)
parser.add_argument("infile", default='D:\pythonproject\\report\data\part1\#civic#_all_civic.180212.csv', nargs='?', help='Input file name, default: data.csv')
parser.add_argument("outfile", nargs='?', help='Output file name, default: [infile_basename].json')
parser.add_argument('-S', '--separator', default=',', help='CSV separator used, default ","')
parser.add_argument('-H', '--headerline', type=int, default=0, help='Header line to use as column names')
parser.add_argument('-c', '--columns', type=int, help='Number of columns to crop to')
parser.add_argument('-u', '--usecols', nargs='+', help='List of names of columns to use')
parser.add_argument('-n', '--names', nargs='+', help='Column names to use')
parser.add_argument('-N', '--nrows', type=int, default=None, help='Number of rows to read')
parser.add_argument('-s', '--skiprows', type=int, default=None, help='Number of rows to skip before reading')
parser.add_argument('-r', '--userows', nargs='+', default=None, help='List of rows to use')
parser.add_argument('-a', '--append', nargs='+', help='Names of columns to append')
parser.add_argument('-p', '--printdata', action='store_true', help='Print formatted data when done')

args = parser.parse_args()

infile = args.infile
if not os.path.isfile(infile):
    print('File "%s" does not exist, aborting. Use --help to show command syntax and command line options.' % infile)
    exit()

outfile = args.outfile
if outfile is None:
    outfile = os.path.splitext(infile)[0] + '.json'

print("Converting %s -> %s [sep=%s, header=%s]" %
      (infile, outfile, args.separator, args.headerline))
if args.usecols is not None:
    print("Only using columns: %s" % (' '.join(args.usecols),))


import pandas as pd
if args.usecols is None:
    try:
        df = pd.read_csv(
            infile, sep=args.separator, header=args.headerline,
            nrows=args.nrows, skiprows=args.skiprows,
            engine='python'
        )
    except Exception:
        print('Could not read CSV data from "%s":\n' % infile)
        raise

else:
    try:
        df = pd.read_csv(
            infile, sep=args.separator, header=args.headerline,
            usecols=args.usecols,
            nrows=args.nrows, skiprows=args.skiprows,
            engine='python'
        )
    except Exception:
        print('Could not read CSV data from "%s":\n' % infile)
        raise

# crop columns
if args.columns:
    df = df[df.columns[:args.columns]]

# remove empty rows
df.dropna(how='all')

# only keep certain rows if given
if args.userows is not None:
    print("Only keeping rows: %s" % (' '.join(args.userows),))
    df = df[df.index.isin([int(row) for row in args.userows])]

# append columns if given
if args.append is not None:
    for additional_column in args.append:
        print('Adding column: %s' % additional_column)
        df[additional_column] = ''

# Change column names if given
if args.names is not None:
    df.columns = args.names

df.to_json(outfile, orient='records')

if args.printdata:
    print(df)

if __name__ == "__main__":
    pass
    




# -*- coding: utf-8 -*-
# !/usr/bin/python
# Create Date 2017/9/29 0029

import os
import json
import xlrd
__author__ = 'huohuo'
base_dir = os.getcwd()

if __name__ == "__main__":
    pass

url = r'D:\pythonproject\report\data\part2\immune_suggestion.xls'
data = xlrd.open_workbook(url)



# -*- coding: utf-8 -*-
# !/usr/bin/python
# Create Date 2018/7/2 0002
__author__ = 'huohuo'
import os
import wx
from win32api import GetSystemMetrics

class Frame(wx.Frame):
    def __init__ (self):
        wx.Frame.__init__(self,None,-1,title="wxApp.",size=(250,250),pos=(0,0))

        #一种方法(wxPython)
        mm=wx.DisplaySize()
        print "width=",mm[0]
        print 'height=',mm[1]
        #另一种方法
        print "width =", GetSystemMetrics (0)
        print "height =",GetSystemMetrics (1)
class App(wx.App):
    def OnInit(self):
        frame = Frame()
        frame.Show(True)
        return True

if __name__ == "__main__":
    app = App(False)
    app.MainLoop()

if __name__ == "__main__":
    pass
    




# -*- coding: utf-8 -*-
# !/usr/bin/python
# Create Date 2018/9/29 0029
__author__ = 'huohuo'
from barcode.writer import ImageWriter
from barcode.codex import Code39
imagewriter = ImageWriter()
#保存到图片中
#add_checksum : Boolean   Add the checksum to code or not (default: True)
ean = Code39("1234567890", writer=imagewriter, add_checksum=False)
# 不需要写后缀，ImageWriter初始化方法中默认self.format = 'PNG'
print '保存到image.png'
ean.save('image')

if __name__ == "__main__":
    pass
    

